uniform sampler2D tex_grass, tex_dirt;
uniform vec3 st_ambient, st_mtl_diffuse, st_mtl_specular, st_light_color;
uniform float st_mtl_alpha;
uniform vec3 st_fog_color;

varying vec3 wpos, vpos, norm, ldir;
varying vec2 texcoord;

void main()
{
	vec2 tc = texcoord * 0.3333;
	float rad = length(wpos.xz);

	// spotlight
	float spot = 1.0 - smoothstep(0.4, 1.2, rad / 4.0);

	// fog
	float fog = 1.0 - smoothstep(1.0, 13.0, -vpos.z);

	// ground texturing
	vec3 grass = texture2D(tex_grass, tc * vec2(16.0, 32.0)).rgb;
	vec3 dirt = texture2D(tex_dirt, tc * 24.0).rgb;

	vec3 snow_color = pow(dirt, vec3(0.15, 0.15, 0.15));
	float snow_cover = smoothstep(1.5, 2.0, rad + grass.g);
	vec3 tex = mix(grass, snow_color, snow_cover);

	// lighting calculations
	vec3 n = normalize(norm);
	vec3 l = normalize(ldir);
	vec3 v = -normalize(vpos);
	vec3 h = normalize(l + v);

	float ndotl = max(dot(n, l), 0.0);
	float ndoth = max(dot(n, h), 0.0);

	vec3 ltcol = st_light_color * spot;

	vec3 diffuse = st_mtl_diffuse * ltcol * ndotl;
	vec3 specular = st_mtl_specular * ltcol * pow(ndoth, 40.0);

	vec3 color = (st_ambient + diffuse) * tex + specular;

	gl_FragColor.rgb = mix(st_fog_color, color, fog);
	gl_FragColor.a = 1.0;
}
