#include <stdio.h>
#include <set>
#include <string>
#include "logger.h"
#include "datapath.h"
#include "assman.h"

static std::set<std::string> paths;

void add_data_path(const char *path)
{
	paths.insert(path);
}

#ifndef TARGET_IPHONE
std::string datafile_path(const char *fname)
{
	std::string res;
	if(!fname) {
		return res;
	}

	std::set<std::string>::const_iterator it = paths.begin();
	while(it != paths.end()) {
		const std::string &path = *it++;
		res = path + "/" + std::string(fname);
		ass_file *fp = ass_fopen(res.c_str(), "r");
		if(fp) {
			ass_fclose(fp);
			return res;
		}
	}

	// It's not found. Return the name itself just in case it's right here
	return std::string(fname);
}
#else
#include <CoreFoundation/CoreFoundation.h>

std::string datafile_path(const char *fname)
{
	std::string res;
	if(!fname) {
		return res;
	}

	CFBundleRef bundle;
	CFURLRef url;
	CFStringRef cfname;

	cfname = CFStringCreateWithCString(0, fname, kCFStringEncodingASCII);

	bundle = CFBundleGetMainBundle();
	if(!(url = CFBundleCopyResourceURL(bundle, cfname, 0, 0))) {
		CFRelease(cfname);
		return fname;
	}
	CFRelease(cfname);

	char path[1024];
	if(!CFURLGetFileSystemRepresentation(url, 1, (unsigned char*)path, sizeof path)) {
		CFRelease(url);
		return fname;
	}
	CFRelease(url);
	return std::string(path);
}
#endif
