#include <assert.h>
#include "gfxutil.h"
#include "mesh.h"
#include "meshgen.h"
#include "texture.h"
#include "shader.h"
#include "sdrman.h"
#include "opengl.h"
#include "logger.h"

void draw_rect(const Vector3 &v1, const Vector3 &v2, Texture2D *tex, ShaderProg *sdr)
{
	static ShaderProg *defsdr;

	if(!defsdr) {
		if(!(defsdr = get_sdrprog("defpost.v.glsl", "defpost.p.glsl"))) {
			static bool didlog;
			if(!didlog) {
				error_log("draw_rect: failed to load default shader\n");
				didlog = true;
			}
			if(!sdr) {
				return;
			}
		}
	}


	if(tex) {
		set_texture(tex);
	}
	if(sdr) {
		sdr->bind();
	} else {
		defsdr->bind();

		assert(defsdr->get_attrib_location("attr_vertex") == MESH_ATTR_VERTEX);
		assert(defsdr->get_attrib_location("attr_texcoord") == MESH_ATTR_TEXCOORD);
	}


	float varr[] = {
		v1.x, v1.y, v1.z, v2.x, v1.y, v2.z,
		v2.x, v2.y, v2.z, v1.x, v2.y, v1.z
	};
	static const float tarr[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
	static const unsigned int idxarr[] = {0, 1, 2, 0, 2, 3};

	// disable VBOs if they are enabled
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	glEnableVertexAttribArray(MESH_ATTR_VERTEX);
	glVertexAttribPointer(MESH_ATTR_VERTEX, 3, GL_FLOAT, GL_FALSE, 0, varr);
	glEnableVertexAttribArray(MESH_ATTR_TEXCOORD);
	glVertexAttribPointer(MESH_ATTR_TEXCOORD, 2, GL_FLOAT, GL_FALSE, 0, tarr);

	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, idxarr);

	glDisableVertexAttribArray(MESH_ATTR_VERTEX);
	glDisableVertexAttribArray(MESH_ATTR_TEXCOORD);

	set_texture(0);
}

static const char *bg_vsdr =
	"attribute vec4 attr_vertex;\n"
	"varying float y;\n"
	"void main() { gl_Position = attr_vertex; y = attr_vertex.y; }\n";

static const char *bg_psdr =
	"uniform vec4 clow, chigh;\n"
	"varying float y;\n"
	"void main()\n"
	"{\n"
	"    float t = max(y * 0.75 + 0.25, 0.0);\n"
	"    gl_FragColor = mix(clow, chigh, t);\n"
	"}\n";

void draw_backdrop(const Vector4 &clow, const Vector4 &chigh)
{
	static float vertex[] = { -1, -1, 1, -1, 1, 1, -1, -1, 1, 1, -1, 1 };
	static int uloc_clow, uloc_chigh;
	static unsigned int vbo;
	static ShaderProg *sdr;

	if(!sdr) {
		sdr = new ShaderProg;
		sdr->create(bg_vsdr, bg_psdr);
		sdr->set_attrib_location("attr_vertex", 0);
		uloc_clow = sdr->get_uniform_location("clow");
		uloc_chigh = sdr->get_uniform_location("chigh");
	}
	if(!vbo) {
		glGenBuffers(1, &vbo);
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glBufferData(GL_ARRAY_BUFFER, sizeof vertex, vertex, GL_STATIC_DRAW);
	} else {
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
	}

	glDisable(GL_DEPTH_TEST);

	sdr->set_uniform(uloc_clow, clow);
	sdr->set_uniform(uloc_chigh, chigh);

	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, 0);
	glDrawArrays(GL_TRIANGLES, 0, 6);
	glDisableVertexAttribArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glEnable(GL_DEPTH_TEST);
}
