#include <vector>
#include <bitset>
#include "input.h"
#include "logger.h"

#define NUM_KEYS	65536
static std::bitset<NUM_KEYS> keystate;

void set_key_state(int key, bool state)
{
	if(key < 0 || key >= NUM_KEYS) {
		error_log("set_key_state: key (%d) out of bounds\n", key);
		return;
	}
	keystate[key] = state;
}

bool get_key_state(int key)
{
	if(key < 0 || key >= NUM_KEYS) {
		error_log("get_key_state: key (%d) out of bounds\n", key);
		return false;
	}
	return keystate[key];
}

struct ButtonState {
	bool pressed;
	int press_x, press_y;
};
static std::vector<ButtonState> bnstate(16);
#define MAX_BNSTATE		128
static int mouse_x, mouse_y;

void set_mouse_state(int x, int y, int bn, bool state)
{
	mouse_x = x;
	mouse_y = y;

	if(bn < 0 || bn >= MAX_BNSTATE) {
		return;
	}
	if((size_t)bn >= bnstate.size()) {
		bnstate.resize(bn + 1);
	}

	bnstate[bn].pressed = state;
	if(state) {
		bnstate[bn].press_x = x;
		bnstate[bn].press_y = y;
	}
}

bool mouse_button(int bn)
{
	if(bn < 0 || (size_t)bn >= bnstate.size()) {
		return false;
	}
	return bnstate[bn].pressed;
}

bool mouse_dragging(int bn)
{
	if(bn < 0 || (size_t)bn >= bnstate.size()) {
		return false;
	}

	return bnstate[bn].pressed && mouse_x != bnstate[bn].press_x &&
		mouse_y != bnstate[bn].press_y;
}

bool mouse_drag_origin(int bn, int *xorig, int *yorig)
{
	if(!mouse_dragging()) {
		// this also handles bound issues, no need to recheck
		*xorig = mouse_x;
		*yorig = mouse_y;
		return false;
	}

	*xorig = bnstate[bn].press_x;
	*yorig = bnstate[bn].press_y;
	return true;
}
