#include "psman.h"

PSysManager::PSysManager()
{
	pslist = 0;
}

PSysManager::~PSysManager()
{
	clear();
}

void PSysManager::clear()
{
	while(pslist) {
		ParticleSystem *tmp = pslist;
		pslist = pslist->next;
		delete tmp;
	}
}

void PSysManager::add(ParticleSystem *ps, long ttl)
{
	ps->ttl = ttl;
	ps->prev = 0;
	ps->next = pslist;
	if(pslist) {
		pslist->prev = ps;
	}
	pslist = ps;
}

void PSysManager::remove(ParticleSystem *ps)
{
	if(pslist == ps) {
		pslist = ps->next;
	}
	if(ps->prev)
		ps->prev->next = ps->next;
	if(ps->next)
		ps->next->prev = ps->prev;
	ps->next = ps->prev = 0;
}

void PSysManager::erase(ParticleSystem *ps)
{
	remove(ps);
	delete ps;
}

void PSysManager::update(long tmsec)
{
	static long prev_tm;
	long dt = tmsec - prev_tm;

	ParticleSystem *ps = pslist;
	while(ps) {
		if(!ps->is_active()) {
			ParticleSystem *tmp = ps;
			ps = ps->next;

			erase(tmp);
			continue;
		}

		ps->ttl -= dt;
		if(ps->ttl <= 0) {
			ps->stop();
		}

		ps->update(tmsec);
		ps = ps->next;
	}
}

void PSysManager::draw() const
{
	ParticleSystem *ps = pslist;
	while(ps) {
		if(ps->is_active()) {
			ps->draw();
		}
		ps = ps->next;
	}
}
