#include <assert.h>
#include "scr_debug.h"
#include "opengl.h"
#include "unistate.h"
#include "sdrman.h"
#include "game.h"
#include "drawtext/drawtext.h"
#include "datapath.h"
#include "logger.h"
#include "timer.h"
#include "scr_game.h"
#include "gfxutil.h"

static ShaderProg *sdr, *color_sdr;
static struct dtx_font *font;

const char *DebugScreen::get_name() const
{
	return "DebugScreen";
}

bool DebugScreen::init()
{
	if(!(sdr = get_sdrprog("font.v.glsl", "font.p.glsl"))) {
		return false;
	}
	int vattr = sdr->get_attrib_location("attr_vertex");
	int tcattr = sdr->get_attrib_location("attr_texcoord");
	assert(vattr != -1 && tcattr != -1);

	dtx_vertex_attribs(vattr, tcattr);

	if(!(font = dtx_open_font_glyphmap(datafile_path("consolas20.glyphmap").c_str()))) {
		error_log("failed to open glyphmap \"consolas20.glyphmap\"\n");
		return false;
	}

	if(!(color_sdr = get_sdrprog("color.v.glsl", "color.p.glsl"))) {
		return false;
	}
	return true;
}

void DebugScreen::cleanup()
{
	delete sdr;
	dtx_close_font(font);
}

void DebugScreen::display() const
{
	OverlayScreen::display();	// this will draw the previous screen

	static unsigned long frame;
	static unsigned long prev_tm;
	unsigned long tm = get_time_msec();
	static char fps_text[64];

	frame++;
	unsigned long interval = tm - prev_tm;
	if(interval >= 1000) {
		float fps = (float)frame / ((float)interval / 1000.0f) + 0.5f;
		sprintf(fps_text, "fps: %d", (int)fps);

		frame = 0;
		prev_tm = tm;
	}

	Matrix4x4 mat;
	mat.set_translation(Vector3(5, get_screen_height() - 20, 0));
	set_world_matrix(mat);

	set_unistate("st_color", Vector4(1, 1, 0, 1));

	sdr->bind();

	dtx_use_font(font, 20);
	dtx_string(fps_text);
	dtx_flush();
}
