#include <vector>
#include "opengl.h"
#include "screen.h"
#include "logger.h"
#include "unistate.h"

static std::vector<BaseScreen*> scrstack;

void push_screen(BaseScreen *screen)
{
	BaseScreen *prev = current_screen();
	if(prev) {
		prev->stop();
	}

	scrstack.push_back(screen);
	screen->start();
}

BaseScreen *pop_screen()
{
	if(scrstack.empty()) {
		error_log("trying to pop a screen, but the stack is empty!\n");
		return 0;
	}
	BaseScreen *res = scrstack.back();
	scrstack.pop_back();

	res->stop();

	BaseScreen *next = current_screen();
	if(next) {
		next->start();
	}
	return res;
}

BaseScreen *current_screen()
{
	if(scrstack.empty()) {
		return 0;
	}
	return scrstack.back();
}

BaseScreen *previous_screen()
{
	int sz = scrstack.size();
	if(sz < 2) {
		return 0;
	}
	return scrstack[sz - 2];
}

BaseScreen::~BaseScreen()
{
	cleanup();
}

const char *BaseScreen::get_name() const
{
	return "BaseScreen";
}

bool BaseScreen::init()
{
	return true;
}

void BaseScreen::cleanup()
{
}

void BaseScreen::start()
{
}

void BaseScreen::stop()
{
}

long BaseScreen::redisplay_interval() const
{
	return 0;	// don't redisplay implicitly by default
}

void BaseScreen::update(unsigned long tmsec)
{
}

void BaseScreen::pre_draw() const
{
}

void BaseScreen::display() const
{
}

void BaseScreen::post_draw() const
{
}

void BaseScreen::keyboard(int key, bool pressed)
{
}

void BaseScreen::motion(int x, int y, bool pressed)
{
}

void BaseScreen::button(int bn, bool pressed, int x, int y)
{
}

void BaseScreen::reshape(int x, int y)
{
}
