#ifndef SCREEN_H_
#define SCREEN_H_

class BaseScreen;

void push_screen(BaseScreen *screen);
BaseScreen *pop_screen();
BaseScreen *current_screen();
BaseScreen *previous_screen();


class BaseScreen {
public:
	virtual ~BaseScreen();

	virtual const char *get_name() const;

	virtual bool init();
	virtual void cleanup();

	/// start will be called just before a screen is going to be activated
	virtual void start();
	/// stop will be called when a screen is being deactivated
	virtual void stop();

	/** each BaseScreen subclass should override this function to specify
	 * the required redisplay interval:
	 * - 0 means no implcit redisplays are to be performed.
	 *   Use request_redisplay to queue up a redisplay.
	 * - Any values > 0 are treated as inter-frame intervals in milliseconds
	 * The default (if not overriden) is 0.
	 */
	virtual long redisplay_interval() const;

	/// update will be called every frame before display
	virtual void update(unsigned long tmsec);

	/// pre_draw will be called before display
	virtual void pre_draw() const;
	/// display will be called whenever a screen needs to redraw itself
	virtual void display() const;
	/// post_draw will be called after display
	virtual void post_draw() const;

	/// keyboard event
	virtual void keyboard(int key, bool pressed);
	/// mouse motion event
	virtual void motion(int x, int y, bool pressed);
	/// mouse button event
	virtual void button(int bn, bool pressed, int x, int y);
	/// reshape event
	virtual void reshape(int x, int y);
};

#endif	// SCREEN_H_
